package edu.iris.dmc.parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.Station;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationIterator;
import edu.iris.dmc.service.StationService;

import edu.iris.dmc.criteria.CriteriaException;

public class XMLParserTest {
	private ServiceUtil util;
	private StationService service;

	@Before
	public void before() {
		util = ServiceUtil.getInstance();
		service = util.getStationService();
	}

	@Test(expected = CriteriaException.class)
	public void testIteratorNet() throws Exception {
		StationIterator iterator = service
				.iterate("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=net&cha=BHZ&loc=*&format=xml");
	}

	@Test
	public void testIteratorSta() throws Exception {
		StationIterator iterator = service
				.iterate("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=sta&cha=BHZ&loc=*&format=xml");

		Network network = null;

		assertNotNull(iterator);

		int antoNumberOfEpocks = 0;
		int anmoNumberOfEpocks = 0;

		while (iterator.hasNext()) {
			Station station = iterator.next();
			if (network == null) {
				network = station.getNetwork();
			}

			if ("ANMO".equals(station.getCode())) {
				anmoNumberOfEpocks++;
			}
			if ("ANTO".equals(station.getCode())) {
				antoNumberOfEpocks++;
			}
		}

		if (iterator != null) {
			iterator.close();
		}
		assertEquals(6, anmoNumberOfEpocks);
		assertEquals(4, antoNumberOfEpocks);
		assertNotNull(network);
		assertEquals("IU", network.getCode());
	}

	@Test
	public void testIteratorCha() throws Exception {
		StationIterator iterator = service
				.iterate("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=cha&cha=BHZ&loc=*&format=xml");

		Network network = null;

		assertNotNull(iterator);

		int antoNumberOfEpocks = 0;
		int anmoNumberOfEpocks = 0;
		int totalNumberOfChannels = 0;

		while (iterator.hasNext()) {
			Station station = iterator.next();
			if (network == null) {
				network = station.getNetwork();
			}

			if ("ANMO".equals(station.getCode())) {
				anmoNumberOfEpocks++;
			}
			if ("ANTO".equals(station.getCode())) {
				antoNumberOfEpocks++;
			}
			if (station.getChannels() != null) {
				totalNumberOfChannels = totalNumberOfChannels
						+ station.getChannels().size();
			}
		}

		if (iterator != null) {
			iterator.close();
		}
		assertEquals(6, anmoNumberOfEpocks);
		assertEquals(4, antoNumberOfEpocks);
		assertEquals(37, totalNumberOfChannels);
		assertNotNull(network);
		assertEquals("IU", network.getCode());
	}

	@Test
	public void testIteratorResp() throws Exception {
		StationIterator iterator = service
				.iterate("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=resp&cha=BHZ&loc=*&format=xml");
	}

	@Test
	public void testFetchNet() throws Exception {
		List<Network> list = service
				.fetch("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=net&cha=BHZ&loc=*&format=xml");

		assertNotNull(list);
		assertEquals(1, list.size());
		Network network = list.get(0);
		assertEquals("IU", network.getCode());
	}

	@Test
	public void testFetchSta() throws Exception {
		List<Network> list = service
				.fetch("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=sta&cha=BHZ&loc=*&format=xml");
		assertNotNull(list);
		assertEquals(1, list.size());
		Network network = list.get(0);
		assertEquals("IU", network.getCode());

		List<Station> stations = network.getStations();
		assertNotNull(stations);
		assertEquals(10, stations.size());

		int antoNumberOfEpocks = 0;
		int anmoNumberOfEpocks = 0;
		for (Station station : stations) {
			if ("ANMO".equals(station.getCode())) {
				anmoNumberOfEpocks++;
			}
			if ("ANTO".equals(station.getCode())) {
				antoNumberOfEpocks++;
			}

		}

		assertEquals(6, anmoNumberOfEpocks);
		assertEquals(4, antoNumberOfEpocks);
	}

	@Test
	public void testFetchCha() throws Exception {
		List<Network> list = service
				.fetch("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=cha&cha=BHZ&loc=*&format=xml");
		assertNotNull(list);
		assertEquals(1, list.size());
		Network network = list.get(0);
		assertEquals("IU", network.getCode());

		List<Station> stations = network.getStations();
		assertNotNull(stations);
		assertEquals(10, stations.size());

		int antoChannels = 0;
		int anmoChannels = 0;
		int totalNumberOfChannels = 0;
		for (Station station : stations) {
			if (station.getChannels() != null) {
				for (Channel c : station.getChannels()) {
					if ("ANTO".equals(station.getCode())) {
						antoChannels++;
					}
					if ("ANMO".equals(station.getCode())) {
						anmoChannels++;
					}
					totalNumberOfChannels++;
				}
			}
		}

		assertEquals(7, antoChannels);
		assertEquals(30, anmoChannels);

		assertEquals(37, totalNumberOfChannels);

	}

	@Test
	public void testFetchResp() throws Exception {
		List<Network> list = service
				.fetch("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=AN*&level=resp&cha=BHZ&loc=*&format=xml");
		assertNotNull(list);
		assertEquals(1, list.size());
		Network network = list.get(0);
		assertEquals("IU", network.getCode());

		List<Station> stations = network.getStations();
		assertNotNull(stations);
		assertEquals(10, stations.size());

		int antoChannels = 0;
		int anmoChannels = 0;
		int totalNumberOfChannels = 0;
		for (Station station : stations) {
			if (station.getChannels() != null) {
				for (Channel c : station.getChannels()) {
					if ("ANTO".equals(station.getCode())) {
						antoChannels++;
					}
					if ("ANMO".equals(station.getCode())) {
						anmoChannels++;
					}
					totalNumberOfChannels++;
				}
			}
		}

		assertEquals(7, antoChannels);
		assertEquals(30, anmoChannels);

		assertEquals(37, totalNumberOfChannels);
	}
}
